"""Various constants to be used in search and related functions, provided as a convenience to help ensure sensible values."""

from .DATASET import DATASET
from .PLATFORM import PLATFORM
from .POLARIZATION import POLARIZATION
from .BEAMMODE import BEAMMODE
from .FLIGHT_DIRECTION import FLIGHT_DIRECTION
from .PRODUCT_TYPE import PRODUCT_TYPE
from .INTERNAL import INTERNAL

#TODO: add INSTRUMENT constants