SENTINEL1 = 'SENTINEL-1'
PALSAR = 'PALSAR'
AVNIR = 'AVNIR-2'
SIRC = 'SIR-C'
S1INSAR = 'SENTINEL-1 InSAR'
SMAP = 'SMAP'
UAVSAR = 'UAVSAR'
RADARSAT1 = 'RADARSAT-1'
ERS = 'ERS'
JERS1 = 'JERS-1'
AIRSAR = 'AIRSAR'
SEASAT = 'SEASAT'